/***********************************************************************
 * File: days.h
 * 	(Extracted from
 *	   "$Id: days.h,v 1.8 2006/07/08 14:22:51 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file defines the is_valid_date() and date_in_English()
 * API-s.
 *
 * Copyright (C) 2005-2006 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _DAYS_H_ /* { */
#define _DAYS_H_

/* include the system definitions */
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

/* for executing "gcc -std=c99..." */
extern int snprintf(char *STR, size_t SIZE, const char *FORMAT,...);


/* The default year of these API is 2000A.D. */
#define DAYS_BASE_YEAR		2000UL


#define IS_LEAP_YEAR(year) (					  \
			((year) % 400 == 0)			  \
			|| ((year) % 100 != 0 && (year) % 4 == 0) \
		)
/*
 * This API determines the numeric string is valid date (yyy...yyymmdd)
 *
 * arguments:
 *	date is pointing to the date string (yyy...yyymmdd).
 *	sz indicates the size of str pointing area.
 *
 * returns:
 *	DAYS_ILLEGAL_ARGS (= -1) : illegal arguments
 *	DAYS_FALSE (= 0)         : the string doesn't means valid date.
 *	DAYS_TRUE  (= 1)         : the string means valid date.
 */
extern int is_valid_date(char *date, size_t sz);
#	define	DAYS_ILLEGAL_ARGS	-1
#	define	DAYS_FALSE		0
#	define	DAYS_TRUE		1


/* type definition for date_in_English() API */
typedef enum days_fmt_t {
	  DAYS_FMT_EUROPEAN
	, DAYS_FMT_AMERICAN
	}	DAYS_FMT;


/*
 * This API translate the numeric string into date in English.
 *
 * arguments:
 *	date is pointing to the numeric string (yyy...yyymmdd).
 *	sz indicates the size of str pointing area.
 *	fmt indicates the format of European or American.
 *	str is pointing the output string.
 *	str_sz indicates the size of the str output area.
 *
 * returns:
 *	NULL : illegal arguments
 *	non-NULL : the translated string address (same as str)
 */
extern char *date_in_English(char *date
		, size_t sz
		, DAYS_FMT fmt
		, char *str
		, size_t str_sz
		);


/*
 * This API returns the day of year.
 *
 * arguments:
 *	date is pointing to the date string.
 *	sz indicates the length of the date string.
 *
 * returns:
 *	zero : illegal arguments
 *	positive integers: the Julian date for date string
 */
int get_day_of_year(char *date, size_t sz);


/*
 * This API counts the days since 1, January AD.1 (it's the 1st day).
 *
 * arguments:
 *	date is pointing to the date.
 *	sz indicates the length of the date string.
 *
 * returns:
 *	LLONG_MIN :		illegal arguments
 *	0LL to 1947821355738LL:	the count of days.
 */
extern long long days_count(char *date, size_t sz);


/*
 * This API subtracts the day-counts of date1 and date2.
 *
 * arguments:
 *	date1 is pointing to the subtracters date string.
 *	sz1 indicates the length of date1 staring.
 *	date2 is pointing to the subtracter date string.
 *	sz2 indicates the length of date2 staring.
 *
 * returns:
 *	LLONG_MIN : illegal arguments
 *	-1947821355738LL to 1947821355738LL: the diff of day-counts.
 */
extern long long diff_dates(char *date1
			, size_t sz1
			, char *date2
			, size_t sz2
			);


/*
 * This API translate day-count to the 'YYY...YYYMMDD' formatted string.
 *
 * arguments:
 *	  day_count indicates the count of past days that returns of
 *	`days_count()' API.
 *	  date points to the buffer that is outputted 'YYY...YYYMMDD'
 *	formatted string.
 *	  sz indicates the size of buffer that is pointed from `date`.
 *	sz must be greater than nine bytes.
 *
 * returns:
 *	NULL: invalid arguments
 *	non-NULL: normal end (same as `date`)
 */
extern char *days_count_to_date_str(long long day_count
				, char *date
				, size_t sz
				);

#endif /* } !defined(_DAYS_H_) */
